# -*- coding: utf-8 -*-
#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
import json

import pytest
import responses

from datarobot import Model
from datarobot.errors import ClientError
from datarobot.models.lift_chart import LiftChart


@pytest.fixture
def lift_chart_validation_data(model_id, lift_chart_bins_data):
    return {"source": "validation", "bins": lift_chart_bins_data, "sourceModelId": model_id}


@pytest.fixture
def multiclass_lift_chart_validation_data(model_id, lift_chart_bins_data):
    return {
        "source": "validation",
        "classBins": [
            {"targetClass": "classA", "bins": lift_chart_bins_data},
            {"targetClass": "classB", "bins": lift_chart_bins_data},
            {"targetClass": "classC", "bins": lift_chart_bins_data},
        ],
    }


@pytest.fixture
def multilabel_lift_chart_validation_data(model_id, project_id, lift_chart_bins_data):
    return get_multilabel_lift_chart_validation_data(model_id, project_id, lift_chart_bins_data)


def get_multilabel_lift_chart_validation_data(model_id, project_id, lift_chart_bins_data):
    return {
        "projectId": project_id,
        "labels": ["cat", u"🐕"],
        "modelId": model_id,
        "source": "validation",
        "labelBins": [
            {"label": "cat", "bins": lift_chart_bins_data},
            {"label": u"🐕", "bins": lift_chart_bins_data},
        ],
    }


@pytest.fixture
def multilabel_lift_chart_validation_parent_data(parent_model_id, project_id, lift_chart_bins_data):
    return get_multilabel_lift_chart_validation_data(
        parent_model_id, project_id, lift_chart_bins_data
    )


@pytest.fixture
def lift_chart_parent_model_validation_data(parent_model_id, lift_chart_bins_data):
    return {"source": "validation", "bins": lift_chart_bins_data, "sourceModelId": parent_model_id}


@pytest.fixture
def lift_chart_parent_model_holdout_data(parent_model_id, lift_chart_bins_data):
    return {"source": "holdout", "bins": lift_chart_bins_data, "sourceModelId": parent_model_id}


@pytest.fixture
def expected_bin_data(lift_chart_validation_data):
    expected_bins = [dict(bin_data) for bin_data in lift_chart_validation_data["bins"]]
    for expected in expected_bins:
        weight = expected.pop("binWeight")
        expected["bin_weight"] = weight
    return expected_bins


def test_instantiation(lift_chart_validation_data, expected_bin_data):
    lc = LiftChart.from_server_data(lift_chart_validation_data)

    assert lc.source == lift_chart_validation_data["source"]
    assert lc.bins == expected_bin_data


def test_future_proof(lift_chart_validation_data):
    data_with_future_keys = dict(lift_chart_validation_data, new_key="some future lift data")
    data_with_future_keys["bins"][0]["new_key"] = "some future bin data"
    LiftChart.from_server_data(data_with_future_keys)


@pytest.fixture
def lift_chart_validation_data_url(project_id, model_id):
    return "https://host_name.com/projects/{}/models/{}/liftChart/validation/".format(
        project_id, model_id
    )


@pytest.fixture
def multiclass_lift_chart_validation_data_url(project_id, model_id):
    return "https://host_name.com/projects/{}/models/{}/multiclassLiftChart/validation/".format(
        project_id, model_id
    )


@pytest.fixture
def multiclass_lift_chart_list_url(project_id, model_id):
    return "https://host_name.com/projects/{}/models/{}/multiclassLiftChart/".format(
        project_id, model_id
    )


@pytest.fixture
def multilabel_lift_chart_validation_data_url(project_id, model_id):
    return "https://host_name.com/projects/{}/models/{}/multilabelLiftCharts/validation/".format(
        project_id, model_id
    )


@pytest.fixture
def multilabel_lift_chart_validation_data_parent_url(project_id, parent_model_id):
    return "https://host_name.com/projects/{}/models/{}/multilabelLiftCharts/validation/".format(
        project_id, parent_model_id
    )


@pytest.fixture
def multilabel_lift_chart_list_url(project_id, model_id):
    return "https://host_name.com/projects/{}/models/{}/multilabelLiftCharts/".format(
        project_id, model_id
    )


@pytest.fixture
def multilabel_lift_chart_parent_list_url(project_id, parent_model_id):
    return "https://host_name.com/projects/{}/models/{}/multilabelLiftCharts/".format(
        project_id, parent_model_id
    )


@pytest.fixture
def lift_chart_parent_model_validation_data_url(project_id, parent_model_id):
    return "https://host_name.com/projects/{}/models/{}/liftChart/validation/".format(
        project_id, parent_model_id
    )


@pytest.fixture
def lift_chart_list_url(project_id, model_id):
    return "https://host_name.com/projects/{}/models/{}/liftChart/".format(project_id, model_id)


@pytest.fixture
def lift_chart_parent_list_url(project_id, parent_model_id):
    return "https://host_name.com/projects/{}/models/{}/liftChart/".format(
        project_id, parent_model_id
    )


@pytest.fixture
def frozen_model_url(project_id, model_id):
    return "https://host_name.com/projects/{}/frozenModels/{}/".format(project_id, model_id)


@responses.activate
def test_get_validation_lift_chart(
    lift_chart_validation_data,
    expected_bin_data,
    lift_chart_validation_data_url,
    project_id,
    model_id,
):
    responses.add(
        responses.GET,
        lift_chart_validation_data_url,
        status=200,
        content_type="application/json",
        body=json.dumps(lift_chart_validation_data),
    )
    model = Model(id=model_id, project_id=project_id)
    lc = model.get_lift_chart("validation")

    assert lc.source == lift_chart_validation_data["source"]
    assert lc.bins == expected_bin_data
    assert lc.source_model_id == model_id


@responses.activate
def test_get_validation_multiclass_lift_chart(
    multiclass_lift_chart_validation_data,
    expected_bin_data,
    multiclass_lift_chart_validation_data_url,
    project_id,
    model_id,
):
    responses.add(
        responses.GET,
        multiclass_lift_chart_validation_data_url,
        status=200,
        content_type="application/json",
        body=json.dumps(multiclass_lift_chart_validation_data),
    )
    model = Model(id=model_id, project_id=project_id)
    lcs = model.get_multiclass_lift_chart("validation")
    # 3-class multiclass contains 3 records for each class
    assert len(lcs) == 3
    for lc in lcs:
        assert lc.source == multiclass_lift_chart_validation_data["source"]
        assert lc.bins == expected_bin_data
        assert lc.source_model_id == model_id


@responses.activate
def test_get_all_multiclass_lift_charts(
    multiclass_lift_chart_validation_data,
    expected_bin_data,
    multiclass_lift_chart_list_url,
    project_id,
    model_id,
):
    responses.add(
        responses.GET,
        multiclass_lift_chart_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"charts": [multiclass_lift_chart_validation_data]}),
    )
    model = Model(id=model_id, project_id=project_id)
    lift_charts = model.get_all_multiclass_lift_charts()

    assert len(lift_charts) == 3
    for lift_chart in lift_charts:
        assert lift_chart.source == multiclass_lift_chart_validation_data["source"]
        assert lift_chart.bins == expected_bin_data
        assert lift_chart.source_model_id == model_id


@responses.activate
def test_get_validation_multilabel_lift_charts(
    multilabel_lift_chart_validation_data,
    expected_bin_data,
    multilabel_lift_chart_validation_data_url,
    project_id,
    model_id,
):
    responses.add(
        responses.GET,
        multilabel_lift_chart_validation_data_url,
        status=200,
        content_type="application/json",
        body=json.dumps(multilabel_lift_chart_validation_data),
    )
    model = Model(id=model_id, project_id=project_id)
    lift_charts = model.get_multilabel_lift_charts("validation")

    expected_labels = ["cat", u"🐕"]

    assert len(lift_charts) == len(expected_labels)
    for lift_chart, expected_label in zip(lift_charts, expected_labels):
        assert lift_chart.source == "validation"
        assert lift_chart.bins == expected_bin_data
        assert lift_chart.source_model_id == model_id
        assert lift_chart.target_class == expected_label


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_get_validation_multilabel_lift_charts_fallback_to_parent_insight(
    multilabel_lift_chart_validation_parent_data,
    expected_bin_data,
    multilabel_lift_chart_validation_data_url,
    multilabel_lift_chart_validation_data_parent_url,
    project_id,
    model_id,
    parent_model_id,
    frozen_json,
    frozen_model_url,
):
    responses.add(responses.GET, multilabel_lift_chart_validation_data_url, status=404)
    responses.add(
        responses.GET,
        frozen_model_url,
        body=frozen_json,
        status=200,
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        multilabel_lift_chart_validation_data_parent_url,
        status=200,
        content_type="application/json",
        body=json.dumps(multilabel_lift_chart_validation_parent_data),
    )
    model = Model(id=model_id, project_id=project_id, is_frozen=True)
    lift_charts = model.get_multilabel_lift_charts("validation", fallback_to_parent_insights=True)

    expected_labels = ["cat", u"🐕"]

    assert len(lift_charts) == len(expected_labels)
    for lift_chart, expected_label in zip(lift_charts, expected_labels):
        assert lift_chart.source == "validation"
        assert lift_chart.bins == expected_bin_data
        assert lift_chart.source_model_id == parent_model_id
        assert lift_chart.target_class == expected_label


@responses.activate
def test_get_validation_multilabel_lift_charts_no_charts_and_fallback_flag_not_set(
    multilabel_lift_chart_validation_data_url, project_id, model_id, parent_model_id,
):
    responses.add(responses.GET, multilabel_lift_chart_validation_data_url, status=404)
    model = Model(id=model_id, project_id=project_id, is_frozen=True)
    with pytest.raises(ClientError):
        model.get_multilabel_lift_charts("validation")


@responses.activate
def test_get_frozen_validation_lift_chart_no_fallback(
    lift_chart_validation_data_url, project_id, model_id
):
    responses.add(responses.GET, lift_chart_validation_data_url, status=404)
    model = Model(id=model_id, project_id=project_id, is_frozen=True)
    with pytest.raises(ClientError):
        model.get_lift_chart("validation")


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_get_frozen_validation_lift_chart_with_fallback(
    lift_chart_parent_model_validation_data,
    expected_bin_data,
    lift_chart_validation_data_url,
    lift_chart_parent_model_validation_data_url,
    frozen_model_url,
    project_id,
    model_id,
    parent_model_id,
    frozen_json,
):

    responses.add(responses.GET, lift_chart_validation_data_url, status=404)
    responses.add(
        responses.GET,
        frozen_model_url,
        body=frozen_json,
        status=200,
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        lift_chart_parent_model_validation_data_url,
        status=200,
        content_type="application/json",
        body=json.dumps(lift_chart_parent_model_validation_data),
    )
    model = Model(id=model_id, project_id=project_id, is_frozen=True)
    lc = model.get_lift_chart("validation", fallback_to_parent_insights=True)

    assert lc.source == lift_chart_parent_model_validation_data["source"]
    assert lc.bins == expected_bin_data
    assert lc.source_model_id == parent_model_id


@responses.activate
def test_get_all_lift_charts(
    lift_chart_validation_data, expected_bin_data, lift_chart_list_url, project_id, model_id
):
    responses.add(
        responses.GET,
        lift_chart_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"charts": [lift_chart_validation_data]}),
    )
    model = Model(id=model_id, project_id=project_id)
    lc_list = model.get_all_lift_charts()

    assert len(lc_list) == 1
    assert lc_list[0].source == lift_chart_validation_data["source"]
    assert lc_list[0].bins == expected_bin_data
    assert lc_list[0].source_model_id == model_id


@responses.activate
def test_get_frozen_all_lift_charts_no_fallback(
    lift_chart_validation_data, expected_bin_data, lift_chart_list_url, project_id, model_id
):
    responses.add(
        responses.GET,
        lift_chart_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"charts": [lift_chart_validation_data]}),
    )
    model = Model(id=model_id, project_id=project_id, is_frozen=True)
    lc_list = model.get_all_lift_charts()

    assert len(lc_list) == 1
    assert lc_list[0].source == lift_chart_validation_data["source"]
    assert lc_list[0].bins == expected_bin_data
    assert lc_list[0].source_model_id == model_id


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_get_frozen_all_lift_charts_with_fallback(
    lift_chart_validation_data,
    lift_chart_parent_model_validation_data,
    lift_chart_parent_model_holdout_data,
    expected_bin_data,
    lift_chart_list_url,
    lift_chart_parent_list_url,
    frozen_model_url,
    project_id,
    model_id,
    parent_model_id,
    frozen_json,
):
    responses.add(
        responses.GET,
        lift_chart_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps({"charts": [lift_chart_validation_data]}),
    )
    responses.add(
        responses.GET,
        frozen_model_url,
        body=frozen_json,
        status=200,
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        lift_chart_parent_list_url,
        status=200,
        content_type="application/json",
        body=json.dumps(
            {
                "charts": [
                    lift_chart_parent_model_validation_data,
                    lift_chart_parent_model_holdout_data,
                ]
            }
        ),
    )
    model = Model(id=model_id, project_id=project_id, is_frozen=True)
    lc_list = model.get_all_lift_charts(fallback_to_parent_insights=True)

    assert len(lc_list) == 2
    assert lc_list[0].source == lift_chart_validation_data["source"]
    assert lc_list[0].bins == expected_bin_data
    assert lc_list[0].source_model_id == model_id

    assert lc_list[1].source == lift_chart_parent_model_holdout_data["source"]
    assert lc_list[1].bins == expected_bin_data
    assert lc_list[1].source_model_id == parent_model_id



#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
import trafaret as t

from datarobot._compat import String
from datarobot.models.api_object import APIObject
from datarobot.models.dataset import Dataset

from ..enums import DEFAULT_MAX_WAIT
from ..utils import encode_utf8_if_py2
from ..utils.waiters import wait_for_async_resolution


class DataEngineQueryGenerator(APIObject):
    """DataEngineQueryGenerator is used to set up time series data prep.

    .. versionadded:: v2.27

    Attributes
    ----------
    id: str
        id of the query generator
    query: str
        text of the generated Spark SQL query
    datasets: list(QueryGeneratorDataset)
        datasets associated with the query generator
    generator_settings: QueryGeneratorSettings
        the settings used to define the query
    generator_type: str
        "TimeSeries" is the only supported type
    """

    _path = "dataEngineQueryGenerators/"
    _converter = t.Dict(
        {
            t.Key("datasets", optional=True): t.List(
                t.Dict(
                    {
                        t.Key("alias"): String,
                        t.Key("dataset_id"): String,
                        t.Key("dataset_version_id", optional=True): String,
                    }
                )
            ),
            t.Key("generator_settings"): t.Dict(
                {
                    t.Key("datetime_partition_column"): String,
                    t.Key("time_unit"): String,
                    t.Key("time_step"): t.Int,
                    t.Key("default_numeric_aggregation_method"): String,
                    t.Key("default_categorical_aggregation_method"): String,
                    t.Key("target", optional=True): String,
                    t.Key("multiseries_id_columns", optional=True): t.List(String),
                    t.Key("default_text_aggregation_method", optional=True): String,
                    t.Key("start_from_series_min_datetime", optional=True): t.Bool,
                    t.Key("end_to_series_max_datetime", optional=True): t.Bool,
                }
            ).allow_extra("*"),
            t.Key("generator_type"): String,
            t.Key("id"): String,
            t.Key("query"): String,
        }
    ).allow_extra("*")

    def __init__(self, **generator_kwargs):
        self.datasets = generator_kwargs["datasets"]
        self.generator_type = generator_kwargs["generator_type"]
        self.generator_settings = generator_kwargs["generator_settings"]
        self.id = generator_kwargs["id"]
        self.query = generator_kwargs["query"]

    def __repr__(self):
        return encode_utf8_if_py2(
            "{}(generator_id={}, datasets={}, generator_type={}, "
            "generator_settings={}, query={}".format(
                self.__class__.__name__,
                self.id,
                self.datasets,
                self.generator_type,
                self.generator_settings,
                self.query,
            )
        )

    @classmethod
    def create(cls, generator_type, datasets, generator_settings):
        """Creates a query generator entity.

        .. versionadded:: v2.27

        Parameters
        ----------
        generator_type : str
            Type of data engine query generator
        datasets : List[QueryGeneratorDataset]
            Source datasets in the Data Engine workspace.
        generator_settings : dict
            Data engine generator settings of the given `generator_type`.

        Returns
        -------
        query_generator : DataEngineQueryGenerator
            The created generator

        Examples
        --------
        .. code-block:: python

            import datarobot as dr
            from datarobot.models.data_engine_query_generator import (
               QueryGeneratorDataset,
               QueryGeneratorSettings,
            )
            dataset = QueryGeneratorDataset(
               alias='My_Awesome_Dataset_csv',
               dataset_id='61093144cabd630828bca321',
               dataset_version_id=1,
            )
            settings = QueryGeneratorSettings(
               datetime_partition_column='date',
               time_unit='DAY',
               time_step=1,
               default_numeric_aggregation_method='sum',
               default_categorical_aggregation_method='mostFrequent',
            )
            g = dr.DataEngineQueryGenerator.create(
               generator_type='TimeSeries',
               datasets=[dataset],
               generator_settings=settings,
            )
            g.id
            >>>'54e639a18bd88f08078ca831'
            g.generator_type
            >>>'TimeSeries'
        """
        data = {
            "generator_type": generator_type,
            "datasets": [d.to_dict() for d in datasets],
            "generator_settings": generator_settings.to_dict(),
        }
        response = cls._client.post(cls._path, data=data)
        finished_url = wait_for_async_resolution(cls._client, response.headers["Location"])
        finished_response = cls._client.get(finished_url)

        return cls.from_server_data(finished_response.json())

    @classmethod
    def get(cls, generator_id):
        """Gets information about a query generator.

        Parameters
        ----------
        generator_id : str
            The identifier of the query generator you want to load.

        Returns
        -------
        query_generator : DataEngineQueryGenerator
            The queried generator

        Examples
        --------
        .. code-block:: python

            import datarobot as dr
            g = dr.DataEngineQueryGenerator.get(generator_id='54e639a18bd88f08078ca831')
            g.id
            >>>'54e639a18bd88f08078ca831'
            g.generator_type
            >>>'TimeSeries'
        """
        path = "{}{}/".format(cls._path, generator_id)
        return cls.from_location(path)

    def create_dataset(self, dataset_id=None, dataset_version_id=None, max_wait=DEFAULT_MAX_WAIT):
        """
        A blocking call that creates a new Dataset from the query generator.
        Returns when the dataset has been successfully processed. If optional
        parameters are not specified the query is applied to the dataset_id
        and dataset_version_id stored in the query generator. If specified they
        will override the stored dataset_id/dataset_version_id, i.e. to prep a
        prediction dataset.

        Parameters
        ----------
        dataset_id: str, optional
            The id of the unprepped dataset to apply the query to
        dataset_version_id: str, optional
            The version_id of the unprepped dataset to apply the query to

        Returns
        -------
        response: Dataset
            The Dataset created from the query generator
        """
        return Dataset.create_from_query_generator(
            self.id, dataset_id, dataset_version_id, max_wait
        )


class QueryGeneratorSettings(object):
    """
    A QueryGenerator settings to be used to create DataEngineQueryGenerator entity.
    .. versionadded:: v2.27
    Attributes
    ----------
    datetime_partition_column : str
        The date column that will be used as a datetime partition column
        in time series project.
    time_unit : str
        Indicates which unit is a basis for time steps of the output dataset.
    time_step : int
        Number of time steps for the output dataset.
    default_numeric_aggregation_method : str
        Default aggregation method used for numeric feature.
    default_categorical_aggregation_method : str
        Default aggregation method used for categorical feature.
    target : str, optional
        The name of target for the output dataset.
    multiseries_id_columns : list[str], optional
        An array with the names of columns identifying the series to which
        row of the output dataset belongs. Currently, only one multiseries
        ID column is supported.
    default_text_aggregation_method : str, optional
        Default aggregation method used for text feature.
    start_from_series_min_datetime : bool, optional
        A boolean value indicating whether post-aggregated series starts from series minimum
        datetime or global minimum datetime.
    end_to_series_max_datetime : bool, optional
        A boolean value indicating whether generates post-aggregated series up to series
        maximum datetime or global maximum datetime.
    Examples
    --------
    .. code-block:: python
        from datarobot.models.data_engine_query_generator import QueryGeneratorSettings
        query_generator_settings = QueryGeneratorSettings(
            datetime_partition_column='date',
            time_unit='DAY',
            time_step=1,
            default_numeric_aggregation_method='sum',
            default_categorical_aggregation_method='mostFrequent',
        )
    """

    def __init__(
        self,
        datetime_partition_column,
        time_unit,
        time_step,
        default_numeric_aggregation_method,
        default_categorical_aggregation_method,
        target=None,
        multiseries_id_columns=None,
        default_text_aggregation_method=None,
        start_from_series_min_datetime=True,
        end_to_series_max_datetime=True,
    ):
        self.datetime_partition_column = datetime_partition_column
        self.time_unit = time_unit
        self.time_step = time_step
        self.default_numeric_aggregation_method = default_numeric_aggregation_method
        self.default_categorical_aggregation_method = default_categorical_aggregation_method
        self.target = target
        self.multiseries_id_columns = multiseries_id_columns
        self.default_text_aggregation_method = default_text_aggregation_method
        self.start_from_series_min_datetime = start_from_series_min_datetime
        self.end_to_series_max_datetime = end_to_series_max_datetime

    def to_dict(self):
        return {
            "datetime_partition_column": self.datetime_partition_column,
            "time_unit": self.time_unit,
            "time_step": self.time_step,
            "default_numeric_aggregation_method": self.default_numeric_aggregation_method,
            "default_categorical_aggregation_method": self.default_categorical_aggregation_method,
            "target": self.target,
            "multiseries_id_columns": self.multiseries_id_columns,
            "default_text_aggregation_method": self.default_text_aggregation_method,
            "start_from_series_min_datetime": self.start_from_series_min_datetime,
            "end_to_series_max_datetime": self.end_to_series_max_datetime,
        }

    def to_payload(self):
        return {
            "datetimePartitionColumn": self.datetime_partition_column,
            "timeUnit": self.time_unit,
            "timeStep": self.time_step,
            "defaultNumericAggregationMethod": self.default_numeric_aggregation_method,
            "defaultCategoricalAggregationMethod": self.default_categorical_aggregation_method,
            "target": self.target,
            "multiseriesIdColumns": self.multiseries_id_columns,
            "defaultTextAggregationMethod": self.default_text_aggregation_method,
            "startFromSeriesMinDatetime": self.start_from_series_min_datetime,
            "endToSeriesMaxDatetime": self.end_to_series_max_datetime,
        }


class QueryGeneratorDataset(object):
    """
    A QueryGenerator dataset to be used to create DataEngineQueryGenerator entity.
    .. versionadded:: v2.27
    Attributes
    ----------
    alias : str
        The name for the dataset.
    dataset_id : str
        The identifier of the dataset.
    dataset_version_id : str, optional
        The version id of the dataset to use.
    Examples
    --------
    .. code-block:: python
        from datarobot.models.data_engine_query_generator import QueryGeneratorDataset
        query_generator_dataset = QueryGeneratorDataset(
            alias='My_Awesome_Dataset_csv',
            dataset_id='61093144cabd630828bca321',
            dataset_version_id=1,
        )
    """

    def __init__(self, alias, dataset_id, dataset_version_id=None):
        self.alias = alias
        self.dataset_id = dataset_id
        self.dataset_version_id = dataset_version_id

    def to_dict(self):
        return {
            "alias": self.alias,
            "dataset_id": self.dataset_id,
            "dataset_version_id": self.dataset_version_id,
        }

    def to_payload(self):
        return {
            "alias": self.alias,
            "datasetId": self.dataset_id,
            "datasetVersionId": self.dataset_version_id,
        }



#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
import json
import os

import responses

from datarobot.models import ComplianceDocumentation


@responses.activate
def test_generate(project_id, one_model, job_id):
    responses.add(
        responses.POST,
        "https://host_name.com/projects/{}/models/{}/complianceDocs/".format(
            project_id, one_model.id
        ),
        status=202,
        adding_headers={
            "Location": "https://host_name.com/projects/{}/jobs/{}/".format(project_id, job_id),
        },
    )
    stub_job_data = {
        "status": "inprogress",
        "url": "https://host_name.com/projects/{}/modelJobs/{}/".format(project_id, job_id),
        "id": job_id,
        "jobType": "model",
        "isBlocked": False,
        "projectId": project_id,
    }
    responses.add(
        responses.GET,
        "https://host_name.com/projects/{}/jobs/{}/".format(project_id, job_id),
        status=200,
        body=json.dumps(stub_job_data),
        content_type="application/json",
    )
    doc = ComplianceDocumentation(project_id, one_model.id)
    job = doc.generate()

    assert job.id == int(job_id)
    assert len(responses.calls) == 2
    post_call, get_call = responses.calls
    assert post_call.request.body is None
    assert get_call.request.url.endswith("{}/".format(job_id))


@responses.activate
def test_download(project_id, one_model, tmpdir):
    url = "https://host_name.com/projects/{}/models/{}/complianceDocs/".format(
        project_id, one_model.id
    )
    responses.add(responses.GET, url, body="bytes and shit", status=200, match_querystring=True)
    doc = ComplianceDocumentation(project_id, one_model.id)
    localpath = tmpdir.join("temp.docx")
    doc.download(localpath.strpath)
    assert os.path.isfile(localpath.strpath)
    with open(localpath.strpath) as f:
        contents = f.read()
        assert contents == "bytes and shit"



#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
import json

import pytest
import responses

from datarobot.errors import ClientError
from datarobot.models import Cluster
from datarobot.utils import from_api


class TestCluster(object):
    def test_instantiation_no_percent(self, cluster_no_percent):
        """
        Given cluster data without optional value
        When I instantiate object
        Then objects fields hold expected data and optional filed is None
        """
        cl = Cluster.from_data(from_api(cluster_no_percent))
        assert cl.name == cluster_no_percent["name"]
        assert cl.percent is None

    def test_instantiation_with_percent(self, cluster_with_percent):
        """
        Given cluster data
        When I instantiate object
        Then objects fields hold expected data
        """
        cl = Cluster.from_data(from_api(cluster_with_percent))
        assert cl.name == cluster_with_percent["name"]
        assert cl.percent == cluster_with_percent["percent"]

    def test_future_proof(self, cluster_with_percent):
        """
        Given cluster data with new key
        When I instantiate object
        Then objects creates without errors and new key is ignored
        """
        new_key = "future"
        data = dict(from_api(cluster_with_percent), new_key=new_key)
        cl = Cluster.from_data(data)
        assert cl.__dict__.get(new_key) is None

    def test_repr(self, cluster_with_percent):
        """
        Given cluster instance
        When I print object using repr
        Then string value should match expected value
        """
        cl = Cluster.from_data(from_api(cluster_with_percent))
        assert str(cl) == "Cluster(name={}, percent={})".format(cl.name, cl.percent)

    # List cluster names
    @responses.activate
    def test_cluster_list_data(self, cluster_url, clusters_data, project_id, model_id):
        """
        Given valid response from api
        When I list cluster names
        Then order and values returned by method should match api response
        """
        responses.add(
            responses.GET,
            cluster_url,
            status=200,
            content_type="application/json",
            body=json.dumps(clusters_data),
        )
        cl = Cluster.list(project_id, model_id)
        assert len(cl) == len(clusters_data["clusters"])
        for idx, cluster in enumerate(cl):
            assert clusters_data["clusters"][idx]["name"] == cluster.name
            assert clusters_data["clusters"][idx]["percent"] == cluster.percent

    @responses.activate
    def test_cluster_list_not_found(self, cluster_url, clusters_data, project_id, model_id):
        """
        Given error response from with status_code 404
        When I list Clusters
        Then method should raise ClientError
        """
        responses.add(
            responses.GET,
            cluster_url,
            status=404,
            content_type="application/json",
            body=json.dumps({"message": "Not Found"}),
        )
        with pytest.raises(ClientError):
            Cluster.list(project_id, model_id)

    # Single cluster name update
    @responses.activate
    def test_update_name(self, cluster_url, clusters_data, project_id, model_id):
        """
        Given valid response from api
        When I update single name
        Then order and values returned by method should match api response
        """
        responses.add(
            responses.PATCH,
            cluster_url,
            status=200,
            content_type="application/json",
            body=json.dumps(clusters_data),
        )
        cl = Cluster.update_name(project_id, model_id, "Cluster 1", "New Name")
        assert len(cl) == len(clusters_data["clusters"])
        for idx, cluster in enumerate(cl):
            assert clusters_data["clusters"][idx]["name"] == cluster.name
            assert clusters_data["clusters"][idx]["percent"] == cluster.percent

    @responses.activate
    @pytest.mark.parametrize(
        "status_code,body",
        [
            (422, {"message": "No current cluster with name Cluster 1"}),
            (404, {"message": "Not Found"}),
        ],
    )
    def test_update_single_name_error(self, cluster_url, project_id, model_id, status_code, body):
        """
        Given error response from api with status_codes (404, 422)
        When I update single cluster name
        Then method should raise ClientError
        """
        responses.add(
            responses.PATCH,
            cluster_url,
            status=status_code,
            content_type="application/json",
            body=json.dumps(body),
        )
        with pytest.raises(ClientError):
            Cluster.update_name(project_id, model_id, "Cluster 1", "New Name")

    # Multiple cluster names update
    @responses.activate
    def test_update_multiple_names_ok(self, cluster_url, clusters_data, project_id, model_id):
        """
        Given valid response from api
        When I update multiple cluster names
        Then order and values returned by method should match api response
        """
        responses.add(
            responses.PATCH,
            cluster_url,
            status=200,
            content_type="application/json",
            body=json.dumps(clusters_data),
        )
        mappings = [
            ("old name 1", "new name 1"),
            ("old name 2", "new name 2"),
            ("old name 3", "new name 3"),
        ]
        cl = Cluster.update_multiple_names(project_id, model_id, mappings)
        assert len(cl) == len(clusters_data["clusters"])
        for idx, cluster in enumerate(cl):
            assert clusters_data["clusters"][idx]["name"] == cluster.name
            assert clusters_data["clusters"][idx]["percent"] == cluster.percent

    @responses.activate
    @pytest.mark.parametrize(
        "status_code,body",
        [
            (422, {"message": "No current cluster with name Cluster 1"}),
            (404, {"message": "Not Found"}),
        ],
    )
    def test_update_multiple_names_error(
        self, cluster_url, project_id, model_id, status_code, body
    ):
        """
        Given error response from api with status_codes (404, 422)
        When I update multiple cluster names
        Then method should raise ClientError
        """
        responses.add(
            responses.PATCH,
            cluster_url,
            status=status_code,
            content_type="application/json",
            body=json.dumps(body),
        )
        mappings = [
            ("old name 1", "new name 1"),
            ("old name 2", "new name 2"),
            ("old name 3", "new name 3"),
        ]
        with pytest.raises(ClientError):
            Cluster.update_multiple_names(project_id, model_id, mappings)



#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
from mock import patch
import pandas
import pytest
import responses

from datarobot import ReasonCodes, ReasonCodesInitialization
from datarobot.models.reason_codes import ReasonCodesPage, ReasonCodesRow


@pytest.fixture
def rci_url(project_collection_url, rci_data):
    return "{}{}/models/{}/reasonCodesInitialization/".format(
        project_collection_url, rci_data["project_id"], rci_data["model_id"]
    )


def test_reason_codes_initialization_future_proof(rci_data):
    ReasonCodesInitialization.from_data(dict(rci_data, future="new"))


def test_reason_codes_future_proof(rc_data):
    ReasonCodes.from_data(dict(rc_data, future="new"))


def test_reason_codes_page_future_proof(rcp_data):
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        ReasonCodesPage.from_data(dict(rcp_data, future="new"))


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_initialization_get_returns_valid_data(rci_url, rci_json, one_rci):
    responses.add(
        responses.GET, rci_url, body=rci_json, status=200, content_type="application/json"
    )
    initialization = ReasonCodesInitialization.get(one_rci.project_id, one_rci.model_id)
    assert isinstance(initialization, ReasonCodesInitialization)
    assert initialization.project_id == one_rci.project_id
    assert initialization.model_id == one_rci.model_id
    assert initialization.reason_codes_sample


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_initialization_create(rci_data, rci_url, project_url, base_job_server_data):
    job_url = "{}jobs/{}/".format(project_url, base_job_server_data["id"])
    responses.add(
        responses.POST,
        rci_url,
        body="",
        status=202,
        content_type="application/json",
        adding_headers={"Location": job_url},
    )
    responses.add(
        responses.GET,
        job_url,
        json=base_job_server_data,
        status=303,
        adding_headers={"Location": rci_url},
    )

    rci_job = ReasonCodesInitialization.create(
        project_id=rci_data["project_id"], model_id=rci_data["model_id"]
    )
    assert str(rci_job.id) == base_job_server_data["id"]


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_initialization_delete(rci_url, rci_data):
    responses.add(responses.DELETE, rci_url, status=200)

    rci = ReasonCodesInitialization.from_data(rci_data)
    rci.delete()


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_get_returns_valid_data(project_id, project_url, rc_json, one_rc):
    url = "{}reasonCodesRecords/{}/".format(project_url, one_rc.id)
    responses.add(responses.GET, url, body=rc_json, status=200, content_type="application/json")
    record = ReasonCodes.get(project_id, one_rc.id)
    assert isinstance(record, ReasonCodes)
    assert record.id == one_rc.id
    assert record.model_id == one_rc.model_id


@responses.activate
def test_reason_codes_page_get_returns_valid_data(project_id, project_url, rcp_json, one_rcp):
    url = "{}reasonCodes/{}/".format(project_url, one_rcp.id)
    responses.add(responses.GET, url, body=rcp_json, status=200, content_type="application/json")
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        record = ReasonCodesPage.get(project_id, one_rcp.id)
    assert isinstance(record, ReasonCodesPage)
    assert record.id == one_rcp.id
    assert record.data == one_rcp.data
    assert record.adjustment_method == "N/A"


@responses.activate
def test_reason_codes_page_get_returns_valid_data_adjusted(
    project_id, project_url, rcp_data_with_adjusted_predictions, one_rcp
):
    url = "{}reasonCodes/{}/".format(project_url, one_rcp.id)
    responses.add(
        responses.GET,
        url,
        json=rcp_data_with_adjusted_predictions,
        status=200,
        content_type="application/json",
    )
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        record = ReasonCodesPage.get(project_id, one_rcp.id)
    assert isinstance(record, ReasonCodesPage)
    assert record.id == one_rcp.id
    assert record.data == one_rcp.data
    assert record.adjustment_method == "exposureNormalized"
    for rc_row in record.data:
        assert rc_row["adjusted_prediction"] == rc_row["prediction"]
        assert rc_row["adjusted_prediction_values"] == rc_row["prediction_values"]


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_create(
    project_id, project_url, project_collection_url, base_job_server_data, rcp_data
):
    rcp_base_url = "{}{}/reasonCodes/".format(project_collection_url, project_id)
    job_url = "{}jobs/{}/".format(project_url, base_job_server_data["id"])
    rcp_url = "{}/{}/".format(rcp_base_url, rcp_data["id"])

    responses.add(
        responses.POST,
        rcp_base_url,
        body="",
        status=202,
        content_type="application/json",
        adding_headers={"Location": job_url},
    )
    responses.add(
        responses.GET,
        job_url,
        json=base_job_server_data,
        status=303,
        adding_headers={"Location": rcp_url},
    )

    rcp_job = ReasonCodes.create(
        project_id=project_id,
        model_id="578e59a41ced2e5a9eb18960",
        dataset_id="581c6256100d2b60586980d9",
    )
    assert str(rcp_job.id) == base_job_server_data["id"]


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_list(project_id, project_url, rc_list_json):
    rc_base_url = "{}reasonCodesRecords/".format(project_url)
    responses.add(
        responses.GET,
        rc_base_url + "?modelId=578e59a41ced2e5a9eb18965",
        status=200,
        body=rc_list_json,
        content_type="application/json",
        match_querystring=True,
    )
    rc_list = ReasonCodes.list(project_id, model_id="578e59a41ced2e5a9eb18965")
    assert len(rc_list) == 3
    for rc in rc_list:
        assert isinstance(rc, ReasonCodes)


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_delete(project_collection_url, project_id, rc_data):
    rc_url = "{}{}/reasonCodesRecords/{}/".format(project_collection_url, project_id, rc_data["id"])
    responses.add(responses.DELETE, rc_url, status=200)

    rc = ReasonCodes.from_data(rc_data)
    rc.delete()


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_iteration(project_id, project_url, one_rcp):
    url = "{}reasonCodes/{}/".format(project_url, one_rcp.id)
    url_page1 = url + "?offset=0&excludeAdjustedPredictions=true"
    url_page2 = url + "?limit=2&offset=2&excludeAdjustedPredictions=true"
    url_page3 = url + "?limit=2&offset=4&excludeAdjustedPredictions=true"
    resp_data_1 = {
        "id": one_rcp.id,
        "count": 5,
        "previous": None,
        "next": url_page2,
        "data": one_rcp.data[:2],
        "reason_codes_record_location": one_rcp.reason_codes_record_location,
    }
    resp_data_2 = dict(resp_data_1, previous=url_page1, next=url_page3, data=one_rcp.data[2:4])
    resp_data_3 = dict(resp_data_1, previous=url_page2, next=None, data=one_rcp.data[4:5])
    responses.add(responses.GET, url_page1, json=resp_data_1, status=200, match_querystring=True)
    responses.add(responses.GET, url_page2, json=resp_data_2, status=200, match_querystring=True)
    responses.add(responses.GET, url_page3, json=resp_data_3, status=200, match_querystring=True)
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        reason_codes_rows = list(
            ReasonCodes(one_rcp.id, project_id, "", "", "", 0, 0, "").get_rows()
        )
    assert len(reason_codes_rows) == 5
    for i, row in enumerate(reason_codes_rows):
        assert isinstance(row, ReasonCodesRow)
        assert row.row_id == one_rcp.data[i]["row_id"]
        assert row.prediction == one_rcp.data[i]["prediction"]
        assert row.prediction_values == one_rcp.data[i]["prediction_values"]
        assert row.reason_codes == one_rcp.data[i]["reason_codes"]


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_as_dataframe_regression(project_url, rcp_data, one_rc):
    url = "{}reasonCodes/{}/".format(project_url, one_rc.id)
    responses.add(responses.GET, url, json=rcp_data, status=200)
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        reason_codes_as_dataframe = one_rc.get_all_as_dataframe()
    assert isinstance(reason_codes_as_dataframe, pandas.DataFrame)
    rows = rcp_data["count"]
    # 5 columns per reason code, plus prediction and row id
    columns = 5 * one_rc.max_codes + 2
    assert reason_codes_as_dataframe.shape == (rows, columns)


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_as_dataframe_regression_with_exposure(
    project_url, one_rc, rcp_data_with_adjusted_predictions
):
    url = "{}reasonCodes/{}/".format(project_url, one_rc.id)
    responses.add(responses.GET, url, json=rcp_data_with_adjusted_predictions, status=200)
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        reason_codes_as_dataframe = one_rc.get_all_as_dataframe()
    assert isinstance(reason_codes_as_dataframe, pandas.DataFrame)
    rows = rcp_data_with_adjusted_predictions["count"]
    # 5 columns per reason code, plus prediction, adjusted prediction and row id
    columns = 5 * one_rc.max_codes + 3
    assert reason_codes_as_dataframe.shape == (rows, columns)


@responses.activate
@pytest.mark.usefixtures("known_warning")
def test_reason_codes_as_dataframe_classification(project_url, rcp_data_classification, one_rc):
    url = "{}reasonCodes/{}/".format(project_url, one_rc.id)
    responses.add(responses.GET, url, json=rcp_data_classification, status=200)
    with patch(
        "datarobot.models.reason_codes.t.URL.check", side_effect=(lambda url, context=None: url)
    ):
        reason_codes_as_dataframe = one_rc.get_all_as_dataframe()
    assert isinstance(reason_codes_as_dataframe, pandas.DataFrame)
    rows = rcp_data_classification["count"]
    # 5 columns per reason code, plus 2 columns per target level, plus row_id and prediction
    columns = (5 * one_rc.max_codes) + (2 * 2) + 2
    assert reason_codes_as_dataframe.shape == (rows, columns)


def test_reason_codes_download_to_csv():
    with patch("datarobot.models.reason_codes.ReasonCodes.get_all_as_dataframe") as rcmock:
        reason_codes = ReasonCodes("fake_id", "fake_project_id", "", "", 3, 0, 0, "")
        reason_codes.download_to_csv("test_file.csv", encoding="ascii")
        rcmock.return_value.to_csv.assert_called_once_with(
            path_or_buf="test_file.csv", header=True, index=False, encoding="ascii"
        )



# -*- encoding: utf-8 -*-
#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
from datetime import datetime
import json

from dateutil.tz import tzutc
import pandas as pd
import pytest
import responses

from datarobot import errors, Predictions
from datarobot.utils import parse_time
from tests.utils import add_response


@pytest.fixture
def sample_prediction():
    return {
        u"positiveClass": None,
        u"task": u"Regression",
        u"predictions": [
            {u"positiveProbability": None, u"prediction": 32.0, u"rowId": 0},
            {u"positiveProbability": None, u"prediction": 100.0, u"rowId": 1},
            {u"positiveProbability": None, u"prediction": 212.0, u"rowId": 2},
        ],
    }


@pytest.fixture
def predictions_regression_with_shap_explanations():
    return {
        "shapBaseValue": [31108.28125],
        "task": "Regression",
        "positiveClass": None,
        "predictions": [
            {
                "positiveProbability": None,
                "predictionExplanationMetadata": {"shapRemainingTotal": -6969.941378326501},
                "prediction": 27229.166666666668,
                "rowId": 0,
                "predictionExplanations": [
                    {
                        "featureValue": "2004",
                        "strength": 3090.8262156678165,
                        "feature": "YearMade",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "999089",
                        "strength": -2788.4580943655205,
                        "feature": "MachineID",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "2006",
                        "strength": -2286.181442643504,
                        "feature": "saledate (Year)",
                        "label": "SalePrice",
                    },
                ],
            },
            {
                "positiveProbability": None,
                "predictionExplanationMetadata": {"shapRemainingTotal": 13439.981135022792},
                "prediction": 60583.333333333336,
                "rowId": 1,
                "predictionExplanations": [
                    {
                        "featureValue": "117657",
                        "strength": 16035.069717943523,
                        "feature": "MachineID",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "77",
                        "strength": 10105.427194858292,
                        "feature": "ModelID",
                        "label": "SalePrice",
                    },
                    {
                        "featureValue": "Medium",
                        "strength": 3432.899846833262,
                        "feature": "ProductSize",
                        "label": "SalePrice",
                    },
                ],
            },
        ],
    }


@pytest.fixture
def predictions_binary_with_shap_explanations():
    return {
        "shapBaseValue": [0.39640626311302185],
        "task": "Binary",
        "positiveClass": 1.0,
        "predictions": [
            {
                "positiveProbability": 0.311178247734139,
                "prediction": 0.0,
                "predictionExplanations": [
                    {
                        "featureValue": "9",
                        "strength": 0.05346482340175079,
                        "feature": "number_diagnoses",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "0",
                        "strength": -0.049024358641319835,
                        "feature": "number_inpatient",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "Surgery-Neuro",
                        "strength": -0.03165299590594699,
                        "feature": "medical_specialty",
                        "label": 1.0,
                    },
                ],
                "rowId": 0,
                "predictionValues": [
                    {"value": 0.3111782477000000, "label": 1.0},
                    {"value": 0.6888217522658611, "label": 0.0},
                ],
                # To test the case with empty metadata from API.
                "predictionExplanationMetadata": {"shapRemainingTotal": None},
                "predictionThreshold": 0.5,
            },
            {
                "positiveProbability": 0.2829861111111111,
                "prediction": 0.0,
                "predictionExplanations": [
                    {
                        "featureValue": "0",
                        "strength": -0.05744066866980976,
                        "feature": "number_inpatient",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "6",
                        "strength": -0.04089034311313498,
                        "feature": "number_diagnoses",
                        "label": 1.0,
                    },
                    {
                        "featureValue": "[50-75)",
                        "strength": 0.03388593271674542,
                        "feature": "weight",
                        "label": 1.0,
                    },
                ],
                "rowId": 1,
                "predictionValues": [
                    {"value": 0.2829861111111111, "label": 1.0},
                    {"value": 0.7170138888888888, "label": 0.0},
                ],
                "predictionExplanationMetadata": {"shapRemainingTotal": -0.00887273281105585},
                "predictionThreshold": 0.5,
            },
        ],
    }


@pytest.fixture
def predictions_multilabel_json():
    predictions = {
        "positiveClass": None,
        "task": "Multilabel",
        "predictions": [
            {
                "predictionValues": [
                    {"threshold": 0.5, "value": 0.6, "label": "cat"},
                    {"threshold": 0.5, "value": 0.5, "label": u"🐕"},
                    {"threshold": 0.5, "value": 0.4, "label": "Maine Coon"},
                    {"threshold": 0.5, "value": 0.7, "label": "British_Shorthair"},
                ],
                "prediction": ["cat", "British_Shorthair"],
                "rowId": 0,
            },
            {
                "predictionValues": [
                    {"threshold": 0.5, "value": 0.1, "label": "cat"},
                    {"threshold": 0.5, "value": 0.4, "label": u"🐕"},
                    {"threshold": 0.5, "value": 0.4, "label": "Maine Coon"},
                    {"threshold": 0.5, "value": 0.1, "label": "British_Shorthair"},
                ],
                "prediction": [],
                "rowId": 1,
            },
        ],
    }
    return predictions


@pytest.fixture
def predictions_with_shap_exp_csv():
    return """row_id,prediction,Explanation_1_feature_name,Explanation_1_feature_value,Explanation_1_strength,shap_remaining_total,shap_base_value
    0,27229.166666666668,YearMade,2004,3090.8262156678165,-6969.941378326501,31108.28125
    1,60583.333333333336,MachineID,117657,16035.069717943523,13439.981135022792,31108.28125
    2,11800.0,fiSecondaryDesc,nan,-14385.515413569377,-4922.765764714323,31108.28125
    3,17453.125,fiSecondaryDesc,nan,-7614.76113549681,-6040.3950427868795,31108.28125
    4,11539.473684210527,Hydraulics_Flow,Standard,-14743.391828540229,-4825.41571693096,31108.28125
    5,22555.555555555555,MachineID,1001274,-6044.702269899164,-2508.024220884521,31108.28125
    6,60583.333333333336,MachineID,772701,20612.841554550116,8862.20929841619,31108.28125
    7,31200.0,MachineID,902002,13999.795343335558,-13908.076521619252,31108.28125
    8,67863.63636363637,YearMade,2008,30190.432159295826,6564.919474920468,31108.28125
    9,21176.470588235294,ProductSize,Large,7127.206390025408,-17059.016865184112,31108.28125
    """


@pytest.fixture
def predictions_metadata_with_shap_exp(prediction_id, project_id, model_id, dataset_id):
    url = "http://localhost/api/v2/projects/{}/predictions/{}/".format(project_id, prediction_id)
    return {
        "url": url,
        "id": prediction_id,
        "projectId": project_id,
        "modelId": model_id,
        "predictionDatasetId": dataset_id,
        "actualValueColumn": None,
        "forecastPoint": None,
        "includesPredictionIntervals": False,
        "predictionIntervalsSize": None,
        "predictionsStartDate": None,
        "predictionsEndDate": None,
        "explanationAlgorithm": "shap",
        "maxExplanations": 1,
        "shapWarnings": {"mismatchRowCount": 1, "maxNormalizedMismatch": 0.01},
    }


@pytest.fixture
def sample_prediction_metadata(prediction_id):
    project_id = "59d92fe8962d7464ca8c6bd6"
    url = "http://localhost/api/v2/projects/{}/predictions/{}/".format(project_id, prediction_id)

    return {
        "projectId": project_id,
        "id": prediction_id,
        "predictionDatasetId": "a_dataset_id",
        "modelId": "a_model_id",
        "includesPredictionIntervals": False,
        "predictionIntervalsSize": None,
        "url": url,
        "forecastPoint": "2017-01-01T15:00:00Z",
        "predictionsStartDate": None,
        "predictionsEndDate": None,
    }


@pytest.fixture
def prediction_id():
    return "59d92fe8962d7464ca8c6bd6"


@pytest.fixture
def model_id():
    return "modelid"


@pytest.fixture
def dataset_id():
    return "datasetid"


@pytest.fixture
def predictions_url(project_id, prediction_id):
    return "https://host_name.com/projects/{}/predictions/{}/".format(project_id, prediction_id)


@pytest.fixture
def predictions_metadata_url(project_id):
    return "https://host_name.com/projects/{}/predictionsMetadata/".format(project_id)


@pytest.fixture
def predictions_metadata_by_id_url(predictions_metadata_url, prediction_id):
    return predictions_metadata_url + "{}/".format(prediction_id)


@responses.activate
def test_get__fetches_metadata(project_id, prediction_id, sample_prediction_metadata):
    metadata_url = "https://host_name.com/projects/{}/predictionsMetadata/{}/".format(
        project_id, prediction_id,
    )
    responses.add(
        responses.GET,
        metadata_url,
        body=json.dumps(sample_prediction_metadata),
        status=200,
        content_type="application/json",
    )

    preds = Predictions.get(project_id, prediction_id)

    assert preds.project_id == project_id
    assert preds.prediction_id == prediction_id
    assert preds.model_id == sample_prediction_metadata["modelId"]
    assert preds.dataset_id == sample_prediction_metadata["predictionDatasetId"]
    assert preds.includes_prediction_intervals == (
        sample_prediction_metadata["includesPredictionIntervals"]
    )
    assert preds.forecast_point == parse_time(sample_prediction_metadata["forecastPoint"])
    assert preds.predictions_start_date == parse_time(
        sample_prediction_metadata["predictionsStartDate"]
    )
    assert preds.predictions_end_date == parse_time(
        sample_prediction_metadata["predictionsEndDate"]
    )
    assert preds.explanation_algorithm is None
    assert preds.max_explanations is None
    assert preds.shap_warnings is None


@responses.activate
def test_get_predictions_with_shap_explanations(
    project_id, prediction_id, predictions_metadata_with_shap_exp, predictions_metadata_by_id_url,
):
    mocked_response = predictions_metadata_with_shap_exp
    add_response(url=predictions_metadata_by_id_url, body=mocked_response)

    predictions = Predictions.get(project_id, prediction_id)

    assert predictions.project_id == project_id
    assert predictions.prediction_id == prediction_id
    assert predictions.model_id == mocked_response["modelId"]
    assert predictions.dataset_id == mocked_response["predictionDatasetId"]
    assert predictions.actual_value_column is None
    assert predictions.forecast_point is None
    assert not predictions.includes_prediction_intervals
    assert predictions.predictions_start_date is None
    assert predictions.predictions_end_date is None
    assert predictions.explanation_algorithm == mocked_response["explanationAlgorithm"]
    assert predictions.max_explanations == mocked_response["maxExplanations"]
    assert predictions.shap_warnings is not None
    assert (
        predictions.shap_warnings["mismatch_row_count"]
        == mocked_response["shapWarnings"]["mismatchRowCount"]
    )
    assert (
        predictions.shap_warnings["max_normalized_mismatch"]
        == mocked_response["shapWarnings"]["maxNormalizedMismatch"]
    )


@responses.activate
def test_list__ok(project_id, model_id, prediction_id, dataset_id, predictions_metadata_url):
    stub = [
        {
            "url": "http://localhost/api/v2/projects/{}/predictions/{}/".format(
                project_id, prediction_id
            ),
            "id": prediction_id,
            "modelId": model_id,
            "predictionDatasetId": dataset_id,
            "includesPredictionIntervals": False,
            "forecastPoint": "2017-01-01T15:00:00Z",
        },
        {
            "url": "http://localhost/api/v2/projects/{}/predictions/{}/".format(
                project_id, prediction_id
            ),
            "id": prediction_id,
            "modelId": model_id,
            "predictionDatasetId": dataset_id,
            "includesPredictionIntervals": False,
            "predictionsStartDate": "2019-05-19T00:00:00.000000Z",
            "predictionsEndDate": "2019-05-20T00:00:00.000000Z",
            "actualValueColumn": "actuals",
        },
        {
            "url": "http://localhost/api/v2/projects/{}/predictions/{}/".format(
                project_id, prediction_id
            ),
            "id": prediction_id,
            "projectId": project_id,
            "modelId": model_id,
            "predictionDatasetId": dataset_id,
            "includesPredictionIntervals": False,
            "predictionsStartDate": None,
            "predictionsEndDate": None,
            "predictionIntervalsSize": None,
            "actualValueColumn": None,
            "forecastPoint": None,
            "explanationAlgorithm": "shap",
            "maxExplanations": 1,
            "shapWarnings": None,
        },
    ]
    add_response(url=predictions_metadata_url, body={"data": stub})

    predictions_list = Predictions.list(project_id)

    assert len(predictions_list) == 3
    prediction = predictions_list[0]
    assert prediction.project_id == project_id
    assert prediction.model_id == model_id
    assert prediction.dataset_id == dataset_id
    assert prediction.prediction_id == prediction_id
    assert not prediction.includes_prediction_intervals
    assert prediction.prediction_intervals_size is None
    assert prediction.forecast_point == datetime(2017, 1, 1, 15, tzinfo=tzutc())
    assert prediction.predictions_start_date is None
    assert prediction.predictions_end_date is None
    assert prediction.explanation_algorithm is None
    assert prediction.max_explanations is None
    assert prediction.shap_warnings is None
    bulk_pred_metadata = predictions_list[1]
    assert bulk_pred_metadata.forecast_point is None
    assert bulk_pred_metadata.predictions_start_date == datetime(2019, 5, 19, tzinfo=tzutc())
    assert bulk_pred_metadata.predictions_end_date == datetime(2019, 5, 20, tzinfo=tzutc())
    assert bulk_pred_metadata.actual_value_column == "actuals"
    assert bulk_pred_metadata.explanation_algorithm is None
    assert bulk_pred_metadata.max_explanations is None
    assert bulk_pred_metadata.shap_warnings is None
    predictions_with_shap_exp = predictions_list[2]
    assert predictions_with_shap_exp.project_id == project_id
    assert predictions_with_shap_exp.model_id == model_id
    assert predictions_with_shap_exp.dataset_id == dataset_id
    assert predictions_with_shap_exp.prediction_id == prediction_id
    assert not predictions_with_shap_exp.includes_prediction_intervals
    assert predictions_with_shap_exp.prediction_intervals_size is None
    assert predictions_with_shap_exp.forecast_point is None
    assert predictions_with_shap_exp.predictions_start_date is None
    assert predictions_with_shap_exp.predictions_end_date is None
    assert predictions_with_shap_exp.explanation_algorithm == "shap"
    assert predictions_with_shap_exp.max_explanations == 1
    assert predictions_with_shap_exp.shap_warnings is None


@responses.activate
def test_get_all_as_dataframe__ok(
    project_id,
    prediction_id,
    project_with_target_json,
    sample_prediction,
    sample_prediction_metadata,
    predictions_url,
    predictions_metadata_by_id_url,
):
    add_response(predictions_url, sample_prediction)
    add_response(predictions_metadata_by_id_url, sample_prediction_metadata)

    obj = Predictions.get(project_id, prediction_id)
    data_frame = obj.get_all_as_dataframe()

    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (len(sample_prediction["predictions"]), 2)
    assert list(data_frame.columns) == [
        "prediction",
        "row_id",
    ]


@responses.activate
def test_get_all_as_dataframe_regression_json_serializer(
    project_id,
    prediction_id,
    predictions_metadata_by_id_url,
    predictions_metadata_with_shap_exp,
    predictions_url,
    predictions_regression_with_shap_explanations,
):
    add_response(predictions_metadata_by_id_url, predictions_metadata_with_shap_exp)
    add_response(predictions_url, predictions_regression_with_shap_explanations)

    predictions = Predictions.get(project_id, prediction_id)
    data_frame = predictions.get_all_as_dataframe(serializer="json")
    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (2, 13)
    assert "row_id" in data_frame.columns
    assert "prediction" in data_frame.columns
    assert "shap_remaining_total" in data_frame.columns
    assert "shap_base_value" in data_frame.columns
    explanation_columns = [column for column in data_frame.columns if "Explanation_" in column]
    assert len(explanation_columns) == 9
    feature_name_columns = [column for column in explanation_columns if "feature_name" in column]
    assert len(feature_name_columns) == 3
    feature_value_columns = [column for column in explanation_columns if "feature_value" in column]
    assert len(feature_value_columns) == 3
    strength_columns = [column for column in explanation_columns if "strength" in column]
    assert len(strength_columns) == 3


@responses.activate
def test_get_all_as_dataframe_binary_json_serializer(
    project_id,
    prediction_id,
    predictions_metadata_by_id_url,
    predictions_metadata_with_shap_exp,
    predictions_url,
    predictions_binary_with_shap_explanations,
):
    add_response(predictions_metadata_by_id_url, predictions_metadata_with_shap_exp)
    add_response(predictions_url, predictions_binary_with_shap_explanations)

    predictions = Predictions.get(project_id, prediction_id)
    data_frame = predictions.get_all_as_dataframe(serializer="json")
    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (2, 18)
    assert "row_id" in data_frame.columns
    assert "prediction" in data_frame.columns
    assert "shap_remaining_total" in data_frame.columns
    assert "shap_base_value" in data_frame.columns
    assert "explained_class" in data_frame.columns
    explanation_columns = [column for column in data_frame.columns if "Explanation_" in column]
    assert len(explanation_columns) == 9
    feature_name_columns = [column for column in explanation_columns if "feature_name" in column]
    assert len(feature_name_columns) == 3
    feature_value_columns = [column for column in explanation_columns if "feature_value" in column]
    assert len(feature_value_columns) == 3
    strength_columns = [column for column in explanation_columns if "strength" in column]
    assert len(strength_columns) == 3


@responses.activate
def test_get_all_as_dataframe_multilabel_json_serializer(
    project_id,
    prediction_id,
    predictions_metadata_by_id_url,
    predictions_url,
    predictions_multilabel_json,
    sample_prediction_metadata,
):
    add_response(predictions_metadata_by_id_url, sample_prediction_metadata)
    add_response(predictions_url, predictions_multilabel_json)

    predictions = Predictions.get(project_id, prediction_id)
    actual = predictions.get_all_as_dataframe(serializer="json")

    expected = pd.DataFrame(
        [
            {
                u"row_id": 0,
                u"threshold_cat": 0.5,
                u"prediction_cat": 1,
                u"class_cat": 0.6,
                u"threshold_🐕": 0.5,
                u"prediction_🐕": 0,
                u"class_🐕": 0.5,
                u"threshold_Maine Coon": 0.5,
                u"prediction_Maine Coon": 0,
                u"class_Maine Coon": 0.4,
                u"threshold_British_Shorthair": 0.5,
                u"prediction_British_Shorthair": 1,
                u"class_British_Shorthair": 0.7,
            },
            {
                u"row_id": 1,
                u"threshold_cat": 0.5,
                u"prediction_cat": 0,
                u"class_cat": 0.1,
                u"threshold_🐕": 0.5,
                u"prediction_🐕": 0,
                u"class_🐕": 0.4,
                u"threshold_Maine Coon": 0.5,
                u"prediction_Maine Coon": 0,
                u"class_Maine Coon": 0.4,
                u"threshold_British_Shorthair": 0.5,
                u"prediction_British_Shorthair": 0,
                u"class_British_Shorthair": 0.1,
            },
        ]
    )
    types_mapping = {}
    for column in expected.columns:
        if column.startswith("prediction_"):
            types_mapping[column] = "int8"
        elif column.startswith("threshold_"):
            types_mapping[column] = "float32"
        elif column.startswith("class_"):
            types_mapping[column] = "float32"
    expected = expected.astype(types_mapping)

    pd.testing.assert_frame_equal(actual, expected[actual.columns])


@responses.activate
def test_get_all_as_dataframe_csv_serializer(
    project_id,
    prediction_id,
    predictions_url,
    predictions_metadata_by_id_url,
    predictions_metadata_with_shap_exp,
    predictions_with_shap_exp_csv,
):
    add_response(url=predictions_metadata_by_id_url, body=predictions_metadata_with_shap_exp)
    add_response(url=predictions_url, body=predictions_with_shap_exp_csv, content_type="text/csv")

    predictions = Predictions.get(project_id, prediction_id)
    data_frame = predictions.get_all_as_dataframe(serializer="csv")
    assert isinstance(data_frame, pd.DataFrame)
    assert data_frame.shape == (10, 7)
    assert "row_id" in data_frame.columns
    assert "prediction" in data_frame.columns
    assert "shap_remaining_total" in data_frame.columns
    assert "shap_base_value" in data_frame.columns
    explanation_columns = [column for column in data_frame.columns if "Explanation_1" in column]
    assert len(explanation_columns) == 3
    assert any(["feature_name" in column for column in explanation_columns])
    assert any(["feature_value" in column for column in explanation_columns])
    assert any(["strength" in column for column in explanation_columns])


@responses.activate
def test_get_not_found(project_id, prediction_id, predictions_metadata_by_id_url):
    add_response(predictions_metadata_by_id_url, "", status=404)
    with pytest.raises(errors.ClientError):
        Predictions.get(project_id, prediction_id)


@responses.activate
def test_get_as_data_frame_server_error(project_id, prediction_id, predictions_metadata_by_id_url):
    add_response(predictions_metadata_by_id_url, "", status=500)
    with pytest.raises(errors.ServerError):
        Predictions.get(project_id, prediction_id)


def test_repr():
    preds = Predictions("project_id", "prediction_id", "model_id", "dataset_id", False)

    assert (
        repr(preds) == "Predictions(prediction_id='prediction_id', project_id='project_id', "
        "model_id='model_id', dataset_id='dataset_id', "
        "includes_prediction_intervals=False, prediction_intervals_size=None, "
        "forecast_point=None, predictions_start_date=None, "
        "predictions_end_date=None, actual_value_column=None, "
        "explanation_algorithm=None, max_explanations=None, shap_warnings=None)"
    )



# coding: utf-8
#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
import json

import pytest
import responses
from trafaret import DataError

from datarobot import CalendarFile, enums, Project, SharingAccess
from datarobot.errors import ClientError, IllegalFileName
from tests.test_helpers import fixture_file_path

_base_url = "https://host_name.com/calendars/"
_valid_calendar_id = "calId"
_invalid_calendar_id = "badCalId"
_valid_calendar_url = "{0}{1}/".format(_base_url, _valid_calendar_id)
_invalid_calendar_url = "{0}{1}/".format(_base_url, _invalid_calendar_id)

calendar_dict = {
    "numEvents": 5,
    "name": "calendar.csv",
    "calendarStartDate": "2018-01-01",
    "created": "2018-07-16T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "calendar.csv",
    "calendarEndDate": "2018-07-04",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": None,
}

calendar_dict_with_name = {
    "numEvents": 5,
    "name": "My Calendar",
    "calendarStartDate": "2018-01-01",
    "created": "2018-07-16T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "calendar.csv",
    "calendarEndDate": "2018-07-04",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": None,
}


multiseries_calendar_dict = {
    "numEvents": 32,
    "name": "multiseries_calendar.csv",
    "calendarStartDate": "2019-01-01",
    "created": "2019-10-17T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "multiseries_calendar.csv",
    "calendarEndDate": "2019-12-26",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": ["series"],
}


multiseries_calendar_dict_with_name = {
    "numEvents": 32,
    "name": "My Multiseries Calendar",
    "calendarStartDate": "2019-01-01",
    "created": "2019-10-17T17:28:13.929687Z",
    "numEventTypes": 5,
    "source": "multiseries_calendar.csv",
    "calendarEndDate": "2019-12-26",
    "projectIds": [],
    "Id": "calId",
    "role": "OWNER",
    "multiseriesIdColumns": ["series"],
}

allowed_country_codes = [
    {"code": "UK", "name": "United Kingdom"},
    {"code": "US", "name": "United States"},
]


@pytest.fixture
def calendar_json():
    return json.dumps(calendar_dict)


@pytest.fixture
def calendar_with_name_json():
    return json.dumps(calendar_dict_with_name)


@pytest.fixture
def multiseries_calendar_json():
    return json.dumps(multiseries_calendar_dict)


@pytest.fixture
def multiseries_calendar_with_name_json():
    return json.dumps(multiseries_calendar_dict_with_name)


@pytest.fixture
def calendar_list_json():
    cal_list = []
    for i in range(1, 11):
        cal_copy = calendar_dict.copy()
        cal_copy["name"] = "cal_name{0}".format(i)
        cal_list.append(cal_copy)
    return json.dumps({"count": len(cal_list), "data": cal_list, "next": None, "previous": None})


@pytest.fixture
def multiseries_calendar_list_json():
    cal_list = []
    for i in range(1, 11):
        cal_copy = multiseries_calendar_dict.copy()
        cal_copy["name"] = "cal_name{0}".format(i)
        cal_list.append(cal_copy)
    return json.dumps({"count": len(cal_list), "data": cal_list, "next": None, "previous": None})


@pytest.fixture
def allowed_countries_list_json():
    return json.dumps(
        {
            "count": len(allowed_country_codes),
            "previous": None,
            "data": allowed_country_codes,
            "next": None,
        }
    )


@pytest.fixture
def calendar_access_control_list_json():
    ac_list = [
        {
            "userId": "ac{0}".format(i),
            "username": "user{0}".format(i),
            "canShare": True,
            "role": "OWNER",
        }
        for i in range(1, 11)
    ]
    return json.dumps({"count": len(ac_list), "data": ac_list, "next": None, "previous": None})


@pytest.fixture
def calendar_invalid_json():
    return json.dumps({"message": "Not Found"})


@responses.activate
def test_calendar_create(calendar_json):
    responses.add(
        responses.POST,
        _base_url + "fileUpload/",
        body="",
        status=202,
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
        content_type="application_json",
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.create(fixture_file_path("calendar.csv"))
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 5
    assert calendar.name == "calendar.csv"
    assert calendar.calendar_start_date == "2018-01-01"
    assert calendar.calendar_end_date == "2018-07-04"
    assert calendar.created == "2018-07-16T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"
    assert not calendar.multiseries_id_columns


def _test_multiseries_calendar_create(multiseries_calendar_json, name=None):
    responses.add(
        responses.POST,
        _base_url + "fileUpload/",
        body="",
        status=202,
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
        content_type="application_json",
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=multiseries_calendar_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.create(
        fixture_file_path("multiseries_calendar.csv"), calendar_name=name
    )
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 32
    assert calendar.name == name if name else "multiseries_calendar.csv"
    assert calendar.calendar_start_date == "2019-01-01"
    assert calendar.calendar_end_date == "2019-12-26"
    assert calendar.created == "2019-10-17T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"
    assert calendar.multiseries_id_columns == ["series"]


@responses.activate
def test_multiseries_calendar_create(multiseries_calendar_json):
    _test_multiseries_calendar_create(multiseries_calendar_json)


@responses.activate
def test_multiseries_calendar_create_with_name(multiseries_calendar_with_name_json):
    _test_multiseries_calendar_create(
        multiseries_calendar_with_name_json, name="My Multiseries Calendar"
    )


def test_calendar_create_non_existent_file():
    with pytest.raises(ValueError):
        CalendarFile.create("bad_file_path")


def test_calendar_create_not_a_string():
    with pytest.raises(ValueError):
        CalendarFile.create({"this definitely": 'isn"t a string'})


def test_calendar_create_multiseries_id_not_a_list():
    with pytest.raises(ValueError) as excinfo:
        CalendarFile.create(
            fixture_file_path("multiseries_calendar.csv"), multiseries_id_columns="bad_id"
        )
    assert str(excinfo.value) == "Expected list of str for multiseries_id_columns, got: bad_id"


@pytest.fixture
def project_without_partition(project_without_target_data):
    project_without_target_data["partition"] = None
    return Project(**project_without_target_data)


@responses.activate
def test_calendar_create_with_name(calendar_with_name_json):
    responses.add(
        responses.POST,
        _base_url + "fileUpload/",
        body="",
        status=202,
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
        content_type="application_json",
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_with_name_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.create(fixture_file_path("calendar.csv"), calendar_name="My Calendar")
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 5
    assert calendar.name == "My Calendar"
    assert calendar.calendar_start_date == "2018-01-01"
    assert calendar.calendar_end_date == "2018-07-04"
    assert calendar.created == "2018-07-16T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"


@responses.activate
def test_calendar_get(calendar_json):
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_json,
        status=200,
        content_type="application/json",
    )
    calendar = CalendarFile.get(_valid_calendar_id)
    assert isinstance(calendar, CalendarFile)
    assert calendar.id == "calId"
    assert calendar.num_events == 5
    assert calendar.name == "calendar.csv"
    assert calendar.calendar_start_date == "2018-01-01"
    assert calendar.calendar_end_date == "2018-07-04"
    assert calendar.created == "2018-07-16T17:28:13.929687Z"
    assert calendar.num_event_types == 5
    assert calendar.project_ids == []
    assert calendar.role == "OWNER"


@responses.activate
def test_create_calendar_from_country_code(calendar_json):
    calendar_params = {
        "countryCode": "US",
        "startDate": "2018-01-01",
        "endDate": "2018-07-04",
    }
    responses.add(
        responses.POST,
        _base_url + "fromCountryCode/",
        body=json.dumps(calendar_params),
        status=202,
        content_type="application/json",
        adding_headers={"Location": "https://host_name.com/status/some-status/"},
    )
    responses.add(
        responses.GET,
        "https://host_name.com/status/some-status/",
        body="",
        status=303,
        adding_headers={"Location": _valid_calendar_url},
        content_type="application/json",
    )
    responses.add(
        responses.GET,
        _valid_calendar_url,
        body=calendar_json,
        status=200,
        content_type="application/json",
    )
    generated_calendar = CalendarFile.create_calendar_from_country_code(
        calendar_params["countryCode"], calendar_params["startDate"], calendar_params["endDate"]
    )
    assert isinstance(generated_calendar, CalendarFile)
    assert generated_calendar.id == "calId"
    assert generated_calendar.role == "OWNER"
    assert generated_calendar.num_events == 5
    assert generated_calendar.name == "calendar.csv"
    assert generated_calendar.calendar_start_date == "2018-01-01"
    assert generated_calendar.calendar_end_date == "2018-07-04"
    assert generated_calendar.created == "2018-07-16T17:28:13.929687Z"
    assert generated_calendar.num_event_types == 5
    assert generated_calendar.project_ids == []
    assert not generated_calendar.multiseries_id_columns


@responses.activate
def test_allowed_countries_list(allowed_countries_list_json):
    responses.add(
        responses.GET,
        "https://host_name.com/calendarCountryCodes/",
        body=allowed_countries_list_json,
    )
    allowed_countries_list = CalendarFile.get_allowed_country_codes()
    assert sorted(allowed_countries_list, key=lambda x: x["name"]) == allowed_country_codes


@responses.activate
def test_calendar_invalid_get(calendar_invalid_json):
    responses.add(responses.GET, _invalid_calendar_url, body=calendar_invalid_json)
    with pytest.raises(DataError):
        CalendarFile.get(_invalid_calendar_id)


@responses.activate
def test_calendar_list(calendar_list_json):
    responses.add(responses.GET, _base_url, body=calendar_list_json)
    cal_list = CalendarFile.list()
    assert len(cal_list) == 10
    for item in cal_list:
        assert isinstance(item, CalendarFile)


@responses.activate
def test_multiseries_calendar_list(multiseries_calendar_list_json):
    responses.add(responses.GET, _base_url, body=multiseries_calendar_list_json)
    cal_list = CalendarFile.list()
    assert len(cal_list) == 10
    for item in cal_list:
        assert isinstance(item, CalendarFile)


@responses.activate
def test_calendar_delete_valid():
    responses.add(responses.DELETE, _valid_calendar_url, status=204)
    assert CalendarFile.delete("calId") is None


@responses.activate
def test_calendar_delete_invalid():
    responses.add(responses.DELETE, _invalid_calendar_url, status=404)
    with pytest.raises(ClientError):
        CalendarFile.delete(_invalid_calendar_id)


@responses.activate
def test_calendar_update_name_valid():
    responses.add(responses.PATCH, _valid_calendar_url, status=200)
    assert CalendarFile.update_name(_valid_calendar_id, "new name") == 200


@responses.activate
def test_calendar_update_name_invalid_calendar():
    responses.add(responses.PATCH, _invalid_calendar_url, status=404)
    with pytest.raises(ClientError):
        CalendarFile.update_name(_invalid_calendar_id, "new name")


@responses.activate
def test_calendar_update_name_invalid_name():
    with pytest.raises(IllegalFileName):
        CalendarFile.update_name(_valid_calendar_id, "£ ¥\n§ ¾")


@responses.activate
def test_calendar_sharing_valid():
    responses.add(responses.PATCH, _valid_calendar_url + "accessControl/", status=200)
    payload = [SharingAccess("valid_username@datarobot.com", enums.SHARING_ROLE.READ_WRITE)]
    assert CalendarFile.share(_valid_calendar_id, payload) == 200


@responses.activate
def test_calendar_sharing_invalid_username():
    responses.add(responses.PATCH, _invalid_calendar_url + "accessControl/", status=422)
    payload = [SharingAccess("invalid_valid_username@datarobot.com", enums.SHARING_ROLE.READ_WRITE)]
    with pytest.raises(ClientError):
        CalendarFile.share(_invalid_calendar_id, payload)


def test_calendar_sharing_invalid_access_list():
    # invalid access_list
    payload = {"look at me, I'm a": "dict"}
    with pytest.raises(AssertionError):
        CalendarFile.share(_valid_calendar_id, payload)
    # invalid items in access_list
    payload = [i for i in range(10)]
    with pytest.raises(AssertionError):
        CalendarFile.share(_valid_calendar_id, payload)


@responses.activate
def test_calendar_sharing_access_list(calendar_access_control_list_json):
    responses.add(
        responses.GET,
        _valid_calendar_url + "accessControl/",
        body=calendar_access_control_list_json,
    )
    access_control_list = CalendarFile.get_access_list(_valid_calendar_id)
    assert len(access_control_list) == 10
    for item in access_control_list:
        assert isinstance(item, SharingAccess)


@responses.activate
def test_calendar_sharing_access_list_user_no_access():
    responses.add(responses.GET, _valid_calendar_url + "accessControl/", status=404)
    with pytest.raises(ClientError):
        CalendarFile.get_access_list(_valid_calendar_id)


def test_retrieve_calendar_id_project_pre_aim(project_without_partition):
    assert project_without_partition.calendar_id is None



#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.
import json

import pytest
import responses

from datarobot import Blueprint
from datarobot.errors import ClientError
from datarobot.models import BlueprintChart, BlueprintTaskDocument, ModelBlueprintChart
from datarobot.utils import from_api


@pytest.fixture
def blueprint_id():
    return "e1c7fc29ba2e612a72272324b8a842af"


@pytest.fixture
def featurelist_id():
    return "e1c7fc29ba2e612a72272324b8a923ba"


@pytest.fixture
def blueprint_data(blueprint_id, project_id):
    return {
        "modelType": "RandomForest Regressor",
        "processes": ["Missing values imputed"],
        "projectId": project_id,
        "id": blueprint_id,
        "blueprintCategory": "DataRobot",
    }


@pytest.fixture
def blueprint_url(project_id, blueprint_id):
    return "https://host_name.com/projects/{}/blueprints/{}/".format(project_id, blueprint_id)


@pytest.fixture
def blueprint_chart_dot():
    return """digraph "Blueprint Chart" {
graph [rankdir=LR]
1 [label="Task1"]
2 [label="Task2"]
1 -> 2
}"""


@pytest.fixture
def blueprint_docs_url(project_id, blueprint_id):
    return "https://host_name.com/projects/{}/blueprints/{}/blueprintDocs/".format(
        project_id, blueprint_id
    )


@pytest.fixture()
def broken_blueprint_docs(blueprint_docs):
    bad_data = {"some_broken_field": "some_broken_value"}
    bad_doc = blueprint_docs[0].copy()
    bad_doc.update(bad_data)
    bad_doc["parameters"][0].update(bad_data)
    bad_doc["references"][0].update(bad_data)
    bad_doc["links"][0].update(bad_data)
    return [bad_doc]


@pytest.fixture
def blueprint_chart_url(project_id, blueprint_id):
    return "https://host_name.com/projects/{}/blueprints/{}/blueprintChart/".format(
        project_id, blueprint_id
    )


def test_instantiation(blueprint_data):
    bp = Blueprint.from_data(from_api(blueprint_data))

    assert bp.model_type == blueprint_data["modelType"]
    assert bp.processes == blueprint_data["processes"]
    assert bp.project_id == blueprint_data["projectId"]
    assert bp.id == blueprint_data["id"]
    assert bp.blueprint_category == blueprint_data["blueprintCategory"]


def test_future_proof(blueprint_data):
    Blueprint.from_data(dict(from_api(blueprint_data), new_key="future"))


def test_non_ascii(blueprint_data, unicode_string):
    bp = Blueprint.from_server_data(dict(blueprint_data, modelType=unicode_string))
    print(bp)  # test that __repr__ works correctly - this used to fail


@responses.activate
def test_blueprint_get(blueprint_data, project_id, blueprint_url, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_data),
    )
    bp = Blueprint.get(project_id, blueprint_id)

    assert bp.model_type == blueprint_data["modelType"]
    assert bp.processes == blueprint_data["processes"]
    assert bp.project_id == blueprint_data["projectId"]
    assert bp.id == blueprint_data["id"]
    assert bp.recommended_featurelist_id is None


@responses.activate
def test_blueprint_get_with_recommended_featurelist_id(
    blueprint_data, project_id, blueprint_url, blueprint_id, featurelist_id
):
    blueprint_data["recommended_featurelist_id"] = featurelist_id
    responses.add(
        responses.GET,
        blueprint_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_data),
    )
    bp = Blueprint.get(project_id, blueprint_id)
    assert bp.id == blueprint_data["id"]
    assert bp.recommended_featurelist_id == featurelist_id


@responses.activate
def test_blueprint_get_not_found(project_id, blueprint_url, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_url,
        status=404,
        content_type="application/json",
        body=json.dumps({"message": "No blueprint data found"}),
    )
    with pytest.raises(ClientError):
        Blueprint.get(project_id, blueprint_id)


@responses.activate
def test_blueprint_chart_get(blueprint_chart_data, project_id, blueprint_chart_url, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_chart_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_chart_data),
    )
    bp = BlueprintChart.get(project_id, blueprint_id)

    assert bp.nodes == blueprint_chart_data["nodes"]
    assert bp.edges == blueprint_chart_data["edges"]


def test_blueprint_chart_to_graphviz(blueprint_chart_data, blueprint_chart_dot):
    bp = BlueprintChart(**blueprint_chart_data)
    assert bp.to_graphviz() == blueprint_chart_dot


@responses.activate
def test_blueprint_get_documents(blueprint_docs_url, blueprint_docs, project_id, blueprint_id):
    responses.add(
        responses.GET,
        blueprint_docs_url,
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_docs),
    )
    bp = Blueprint.from_data({"id": blueprint_id, "project_id": project_id})
    docs = bp.get_documents()
    assert len(docs) == 1
    assert isinstance(docs[0], BlueprintTaskDocument)
    assert docs[0].title == blueprint_docs[0]["title"]
    assert docs[0].task == blueprint_docs[0]["task"]
    assert docs[0].description == blueprint_docs[0]["description"]
    assert docs[0].links == blueprint_docs[0]["links"]
    assert docs[0].parameters == blueprint_docs[0]["parameters"]
    assert docs[0].references == blueprint_docs[0]["references"]


@responses.activate
def test_blueprint_get_documents_future_proof(
    blueprint_docs_url, broken_blueprint_docs, project_id, blueprint_id
):
    responses.add(
        responses.GET,
        blueprint_docs_url,
        status=200,
        content_type="application/json",
        body=json.dumps(broken_blueprint_docs),
    )
    bp = Blueprint.from_data({"id": blueprint_id, "project_id": project_id})
    docs = bp.get_documents()
    assert len(docs) == 1
    assert isinstance(docs[0], BlueprintTaskDocument)


@responses.activate
def test_model_blueprint_chart_get(blueprint_chart_data, project_id, model_id):
    responses.add(
        responses.GET,
        "https://host_name.com/projects/{}/models/{}/blueprintChart/".format(project_id, model_id),
        status=200,
        content_type="application/json",
        body=json.dumps(blueprint_chart_data),
    )
    model_bp = ModelBlueprintChart.get(project_id, model_id)

    assert model_bp.nodes == blueprint_chart_data["nodes"]
    assert model_bp.edges == blueprint_chart_data["edges"]



#
# Copyright 2021 DataRobot, Inc. and its affiliates.
#
# All rights reserved.
#
# DataRobot, Inc.
#
# This is proprietary source code of DataRobot, Inc. and its
# affiliates.
#
# Released under the terms of DataRobot Tool and Utility Agreement.


def enum(*vals, **enums):
    """
    Enum without third party libs and compatible with py2 and py3 versions.
    """
    enums.update(dict(zip(vals, vals)))
    return type("Enum", (), enums)


CV_METHOD = enum(
    DATETIME="datetime", GROUP="group", RANDOM="random", STRATIFIED="stratified", USER="user",
)


SCORING_TYPE = enum(cross_validation="crossValidation", validation="validation")


DATETIME_AUTOPILOT_DATA_SELECTION_METHOD = enum(DURATION="duration", ROW_COUNT="rowCount")
DATETIME_AUTOPILOT_DATA_SAMPLING_METHOD = enum(LATEST="latest", RANDOM="random")


VERBOSITY_LEVEL = enum(SILENT=0, VERBOSE=2)


# This is deprecated, to be removed in 3.0.
MODEL_JOB_STATUS = enum(ERROR="error", INPROGRESS="inprogress", QUEUE="queue")


# This is the job/queue status enum we want to keep.
# In 3.0 this will be INITIALIZED, RUNNING, ABORTED, COMPLETED, ERROR.
# And maybe the name will change to JobStatus.
QUEUE_STATUS = enum(
    ABORTED="ABORTED",
    COMPLETED="COMPLETED",
    ERROR="error",
    INPROGRESS="inprogress",
    QUEUE="queue",
    RUNNING="RUNNING",
    INITIALIZING="INITIALIZING",
)


AUTOPILOT_MODE = enum(
    FULL_AUTO="auto", MANUAL="manual", QUICK="quick", COMPREHENSIVE="comprehensive",
)


PROJECT_STAGE = enum(
    AIM="aim", EDA="eda", EDA2="eda2", FASTEDA="fasteda", EMPTY="empty", MODELING="modeling"
)


ASYNC_PROCESS_STATUS = enum(
    ABORTED="ABORTED",
    COMPLETED="COMPLETED",
    ERROR="ERROR",
    INITIALIZED="INITIALIZED",
    RUNNING="RUNNING",
)


LEADERBOARD_SORT_KEY = enum(PROJECT_METRIC="metric", SAMPLE_PCT="samplePct")


TARGET_TYPE = enum(
    BINARY="Binary",
    MULTICLASS="Multiclass",
    MULTILABEL="Multilabel",
    REGRESSION="Regression",
    UNSTRUCTURED="Unstructured",
    ANOMALY="Anomaly",
)


JOB_TYPE = enum(
    BATCH_PREDICTIONS="batchPredictions",
    BATCH_PREDICTION_JOB_DEFINITIONS="batchPredictionJobDefinitions",
    FEATURE_IMPACT="featureImpact",
    FEATURE_EFFECTS="featureEffects",
    FEATURE_FIT="featureFit",
    MODEL="model",
    MODEL_EXPORT="modelExport",
    PREDICT="predict",
    TRAINING_PREDICTIONS="trainingPredictions",
    PRIME_MODEL="primeModel",
    PRIME_RULESETS="primeRulesets",
    PRIME_VALIDATION="primeDownloadValidation",
    REASON_CODES="reasonCodes",
    REASON_CODES_INITIALIZATION="reasonCodesInitialization",
    PREDICTION_EXPLANATIONS="predictionExplanations",
    PREDICTION_EXPLANATIONS_INITIALIZATION="predictionExplanationsInitialization",
    RATING_TABLE_VALIDATION="validateRatingTable",
    SHAP_IMPACT="shapImpact",
)


PREDICT_JOB_STATUS = enum(ABORTED="ABORTED", ERROR="error", INPROGRESS="inprogress", QUEUE="queue")

PREDICTION_PREFIX = enum(DEFAULT="class_")

PRIME_LANGUAGE = enum(JAVA="Java", PYTHON="Python")

# CATEGORICAL is deprecated, to be removed in 2.22
VARIABLE_TYPE_TRANSFORM = enum(
    CATEGORICAL="categorical", CATEGORICAL_INT="categoricalInt", NUMERIC="numeric", TEXT="text",
)


DATE_EXTRACTION = enum(
    MONTH="month",
    MONTH_DAY="monthDay",
    WEEK="week",
    WEEK_DAY="weekDay",
    YEAR="year",
    YEAR_DAY="yearDay",
)


POSTGRESQL_DRIVER = enum(ANSI="PostgreSQL ANSI", UNICODE="PostgreSQL Unicode")

BLENDER_METHOD = enum(
    AVERAGE="AVG",
    ENET="ENET",
    GLM="GLM",
    MAE="MAE",
    MAEL1="MAEL1",
    MEDIAN="MED",
    PLS="PLS",
    RANDOM_FOREST="RF",
    LIGHT_GBM="LGBM",
    TENSORFLOW="TF",
    FORECAST_DISTANCE_ENET="FORECAST_DISTANCE_ENET",
    FORECAST_DISTANCE_AVG="FORECAST_DISTANCE_AVG",
)

TS_BLENDER_METHOD = enum(
    AVERAGE="AVG",
    MEDIAN="MED",
    FORECAST_DISTANCE_ENET="FORECAST_DISTANCE_ENET",
    FORECAST_DISTANCE_AVG="FORECAST_DISTANCE_AVG",
)

CHART_DATA_SOURCE = enum(
    CROSSVALIDATION="crossValidation", HOLDOUT="holdout", VALIDATION="validation",
)


SCALEOUT_MODELING_MODE = enum(
    AUTOPILOT="autopilot", DISABLED="disabled", REPOSITORY_ONLY="repositoryOnly",
)


DATA_SUBSET = enum(
    ALL="all",
    VALIDATION_AND_HOLDOUT="validationAndHoldout",
    HOLDOUT="holdout",
    ALL_BACKTESTS="allBacktests",
)

FEATURE_TYPE = enum(NUMERIC="numeric", CATEGORICAL="categorical", DATETIME="datetime")

DEFAULT_MAX_WAIT = 600

# default time out values in seconds for waiting response from client
DEFAULT_TIMEOUT = enum(
    CONNECT=6.05,  # time in seconds for the connection to server to be established
    READ=60,  # time in seconds after which to conclude the server isn't responding anymore
    UPLOAD=600,  # time in seconds after which to conclude that project dataset cannot be uploaded
)

# Time in seconds after which to conclude the server isn't responding anymore
# same as in DEFAULT_TIMEOUT, keeping for backwards compatibility
DEFAULT_READ_TIMEOUT = DEFAULT_TIMEOUT.READ

TARGET_LEAKAGE_TYPE = enum(
    SKIPPED_DETECTION="SKIPPED_DETECTION",
    FALSE="FALSE",
    MODERATE_RISK="MODERATE_RISK",
    HIGH_RISK="HIGH_RISK",
)

TREAT_AS_EXPONENTIAL = enum(ALWAYS="always", NEVER="never", AUTO="auto")

DIFFERENCING_METHOD = enum(AUTO="auto", SIMPLE="simple", NONE="none", SEASONAL="seasonal")

TIME_UNITS = enum(
    MILLISECOND="MILLISECOND",
    SECOND="SECOND",
    MINUTE="MINUTE",
    HOUR="HOUR",
    DAY="DAY",
    WEEK="WEEK",
    MONTH="MONTH",
    QUARTER="QUARTER",
    YEAR="YEAR",
)

PERIODICITY_MAX_TIME_STEP = 9223372036854775807

RECOMMENDED_MODEL_TYPE = enum(
    MOST_ACCURATE="Most Accurate",
    FAST_ACCURATE="Fast & Accurate",
    RECOMMENDED_FOR_DEPLOYMENT="Recommended for Deployment",
)

AVAILABLE_STATEMENT_TYPES = enum(
    INSERT="insert", UPDATE="update", INSERT_UPDATE="insert_update", CREATE_TABLE="create_table",
)


class _DEPLOYMENT_HEALTH_STATUS(object):
    PASSING = "passing"
    WARNING = "warning"
    FAILING = "failing"
    UNKNOWN = "unknown"

    ALL = [PASSING, WARNING, FAILING, UNKNOWN]


class DEPLOYMENT_SERVICE_HEALTH_STATUS(_DEPLOYMENT_HEALTH_STATUS):
    pass


class DEPLOYMENT_MODEL_HEALTH_STATUS(_DEPLOYMENT_HEALTH_STATUS):
    pass


class DEPLOYMENT_ACCURACY_HEALTH_STATUS(_DEPLOYMENT_HEALTH_STATUS):
    UNAVAILABLE = "unavailable"

    ALL = _DEPLOYMENT_HEALTH_STATUS.ALL + [UNAVAILABLE]


class DEPLOYMENT_EXECUTION_ENVIRONMENT_TYPE(object):
    DATAROBOT = "datarobot"
    EXTERNAL = "external"

    ALL = [DATAROBOT, EXTERNAL]


class DEPLOYMENT_IMPORTANCE(object):
    CRITICAL = "CRITICAL"
    HIGH = "HIGH"
    MODERATE = "MODERATE"
    LOW = "LOW"

    ALL = [CRITICAL, HIGH, MODERATE, LOW]


SERIES_AGGREGATION_TYPE = enum(AVERAGE="average", TOTAL="total")

MONOTONICITY_FEATURELIST_DEFAULT = object()

SERIES_ACCURACY_ORDER_BY = enum(
    MULTISERIES_VALUE="multiseriesValue",
    ROW_COUNT="rowCount",
    VALIDATION_SCORE="validationScore",
    BACKTESTING_SCORE="backtestingScore",
    HOLDOUT_SCORE="holdoutScore",
)

SHARING_ROLE = enum(
    OWNER="OWNER",
    READ_WRITE="READ_WRITE",
    USER="USER",
    EDITOR="EDITOR",
    READ_ONLY="READ_ONLY",
    CONSUMER="CONSUMER",
)

MODEL_REPLACEMENT_REASON = enum(
    ACCURACY="ACCURACY",
    DATA_DRIFT="DATA_DRIFT",
    ERRORS="ERRORS",
    SCHEDULED_REFRESH="SCHEDULED_REFRESH",
    SCORING_SPEED="SCORING_SPEED",
    OTHER="OTHER",
)


EXPLANATIONS_ALGORITHM = enum(SHAP="shap")


class FEATURE_ASSOCIATION_TYPE(object):
    ASSOCIATION = "association"
    CORRELATION = "correlation"

    ALL = [ASSOCIATION, CORRELATION]


class FEATURE_ASSOCIATION_METRIC(object):
    # association
    MUTUAL_INFO = "mutualInfo"
    CRAMER = "cramersV"
    # correlation
    SPEARMAN = "spearman"
    PEARSON = "pearson"
    TAU = "tau"

    ALL = [MUTUAL_INFO, CRAMER, SPEARMAN, PEARSON, TAU]


class SERVICE_STAT_METRIC(object):
    TOTAL_PREDICTIONS = "totalPredictions"
    TOTAL_REQUESTS = "totalRequests"
    SLOW_REQUESTS = "slowRequests"
    EXECUTION_TIME = "executionTime"
    RESPONSE_TIME = "responseTime"
    USER_ERROR_RATE = "userErrorRate"
    SERVER_ERROR_RATE = "serverErrorRate"
    NUM_CONSUMERS = "numConsumers"
    CACHE_HIT_RATIO = "cacheHitRatio"
    MEDIAN_LOAD = "medianLoad"
    PEAK_LOAD = "peakLoad"

    ALL = [
        TOTAL_PREDICTIONS,
        TOTAL_REQUESTS,
        SLOW_REQUESTS,
        EXECUTION_TIME,
        RESPONSE_TIME,
        USER_ERROR_RATE,
        SERVER_ERROR_RATE,
        NUM_CONSUMERS,
        CACHE_HIT_RATIO,
        MEDIAN_LOAD,
        PEAK_LOAD,
    ]


class DATA_DRIFT_METRIC(object):
    PSI = "psi"
    KL_DIVERGENCE = "kl_divergence"
    DISSIMILARITY = "dissimilarity"
    HELLINGER = "hellinger"
    JS_DIVERGENCE = "js_divergence"
    ALL = [PSI, KL_DIVERGENCE, DISSIMILARITY, HELLINGER, JS_DIVERGENCE]


class ACCURACY_METRIC(object):
    ACCURACY = "Accuracy"
    AUC = "AUC"
    BALANCED_ACCURACY = "Balanced Accuracy"
    FVE_BINOMIAL = "FVE Binomial"
    GINI_NORM = "Gini Norm"
    KOLMOGOROV_SMIRNOV = "Kolmogorov-Smirnov"
    LOGLOSS = "LogLoss"
    RATE_TOP5 = "Rate@Top5%"
    RATE_TOP10 = "Rate@Top10%"

    GAMMA_DEVIANCE = "Gamma Deviance"
    FVE_GAMMA = "FVE Gamma"
    FVE_POISSON = "FVE Poisson"
    FVE_TWEEDIE = "FVE Tweedie"
    MAD = "MAD"
    MAE = "MAE"
    MAPE = "MAPE"
    POISSON_DEVIANCE = "Poisson Deviance"
    R_SQUARED = "R Squared"
    RMSE = "RMSE"
    RMSLE = "RMSLE"
    TWEEDIE_DEVIANCE = "Tweedie Deviance"

    ALL_CLASSIFICATION = [
        ACCURACY,
        AUC,
        BALANCED_ACCURACY,
        FVE_BINOMIAL,
        GINI_NORM,
        KOLMOGOROV_SMIRNOV,
        LOGLOSS,
        RATE_TOP5,
        RATE_TOP10,
    ]
    ALL_REGRESSION = [
        GAMMA_DEVIANCE,
        FVE_GAMMA,
        FVE_POISSON,
        FVE_TWEEDIE,
        MAD,
        MAE,
        MAPE,
        POISSON_DEVIANCE,
        R_SQUARED,
        RMSE,
        RMSLE,
        TWEEDIE_DEVIANCE,
    ]
    ALL = [
        ACCURACY,
        AUC,
        BALANCED_ACCURACY,
        FVE_BINOMIAL,
        GINI_NORM,
        KOLMOGOROV_SMIRNOV,
        LOGLOSS,
        RATE_TOP5,
        RATE_TOP10,
        GAMMA_DEVIANCE,
        FVE_GAMMA,
        FVE_POISSON,
        FVE_TWEEDIE,
        MAD,
        MAE,
        MAPE,
        POISSON_DEVIANCE,
        R_SQUARED,
        RMSE,
        RMSLE,
        TWEEDIE_DEVIANCE,
    ]


class EXECUTION_ENVIRONMENT_VERSION_BUILD_STATUS(object):
    """Enum of possible build statuses of execution environment version."""

    SUBMITTED = "submitted"
    PROCESSING = "processing"
    FAILED = "failed"
    SUCCESS = "success"

    FINAL_STATUSES = [FAILED, SUCCESS]


class CUSTOM_MODEL_IMAGE_TYPE(object):
    """Enum of types that can represent a custom model image"""

    CUSTOM_MODEL_VERSION = "customModelVersion"
    CUSTOM_MODEL_IMAGE = "customModelImage"

    ALL = [CUSTOM_MODEL_IMAGE, CUSTOM_MODEL_VERSION]


class _SHARED_TARGET_TYPE(object):
    """Enum of all target types shared by tasks and models"""

    BINARY = "Binary"
    ANOMALY = "Anomaly"
    REGRESSION = "Regression"
    MULTICLASS = "Multiclass"
    ALL = [BINARY, ANOMALY, REGRESSION, MULTICLASS]


class CUSTOM_MODEL_TARGET_TYPE(_SHARED_TARGET_TYPE):
    """Enum of valid custom model target types"""

    UNSTRUCTURED = "Unstructured"
    REQUIRES_TARGET_NAME = ("Binary", "Multiclass", "Regression")

    ALL = _SHARED_TARGET_TYPE.ALL + [UNSTRUCTURED]


class CUSTOM_TASK_TARGET_TYPE(_SHARED_TARGET_TYPE):
    """Enum of valid custom task target types"""

    TRANSFORM = "Transform"

    ALL = _SHARED_TARGET_TYPE.ALL + [TRANSFORM]


class NETWORK_EGRESS_POLICY(object):
    """Enum of valid network egress policy"""

    NONE = "NONE"
    PUBLIC = "PUBLIC"

    ALL = [NONE, PUBLIC]


class SOURCE_TYPE(object):
    """Enum of backtest source types"""

    TRAINING = "training"
    VALIDATION = "validation"

    ALL = [TRAINING, VALIDATION]


class DATETIME_TREND_PLOTS_STATUS(object):
    COMPLETED = "completed"
    NOT_COMPLETED = "notCompleted"
    IN_PROGRESS = "inProgress"
    ERRORED = "errored"
    NOT_SUPPORTED = "notSupported"
    INSUFFICIENT_DATA = "insufficientData"

    ALL = [COMPLETED, NOT_COMPLETED, IN_PROGRESS, ERRORED, NOT_SUPPORTED, INSUFFICIENT_DATA]


class DATETIME_TREND_PLOTS_RESOLUTION(object):
    MILLISECONDS = "milliseconds"
    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"
    QUARTERS = "quarters"
    YEARS = "years"

    ALL = (
        MILLISECONDS,
        SECONDS,
        MINUTES,
        HOURS,
        DAYS,
        WEEKS,
        MONTHS,
        QUARTERS,
        YEARS,
    )


SNAPSHOT_POLICY = enum(SPECIFIED="specified", LATEST="latest", DYNAMIC="dynamic")


class AllowedTimeUnitsSAFER(object):
    """Enum for SAFER allowed time units"""

    MILLISECOND = "MILLISECOND"
    SECOND = "SECOND"
    MINUTE = "MINUTE"
    HOUR = "HOUR"
    DAY = "DAY"
    WEEK = "WEEK"
    MONTH = "MONTH"
    QUARTER = "QUARTER"
    YEAR = "YEAR"

    ALL = (MILLISECOND, SECOND, MINUTE, HOUR, DAY, WEEK, MONTH, QUARTER, YEAR)


class FeatureDiscoveryMode(object):
    DEFAULT = "default"
    MANUAL = "manual"
    ALL = (DEFAULT, MANUAL)


class AnomalyAssessmentStatus(object):
    COMPLETED = "completed"
    NO_DATA = "noData"  # when there is no series in backtest/source
    NOT_SUPPORTED = "notSupported"  # when full training subset can not be fit into memory.

    ALL = (COMPLETED, NO_DATA, NOT_SUPPORTED)


class UnsupervisedTypeEnum(object):
    ANOMALY = "anomaly"
    CLUSTERING = "clustering"


class FairnessMetricsSet(object):
    PROPORTIONAL_PARITY = "proportionalParity"
    EQUAL_PARITY = "equalParity"
    PREDICTION_BALANCE = "predictionBalance"
    TRUE_FAVORABLE_AND_UNFAVORABLE_RATE_PARITY = "trueFavorableAndUnfavorableRateParity"
    FAVORABLE_AND_UNFAVORABLE_PREDICTIVE_VALUE_PARITY = (
        "favorableAndUnfavorablePredictiveValueParity"
    )

    ALL = (
        PROPORTIONAL_PARITY,
        EQUAL_PARITY,
        PREDICTION_BALANCE,
        TRUE_FAVORABLE_AND_UNFAVORABLE_RATE_PARITY,
        FAVORABLE_AND_UNFAVORABLE_PREDICTIVE_VALUE_PARITY,
    )
